//---------------------------------------------------------------------------
#include <stdio.h>
#include <string.h>
#include <vcl.h>
#pragma hdrstop

//---------------------------------------------------------------------------
typedef unsigned char uchar;

#define usage() printf("usage: img2vhd_generator.exe source_path.bmp\n\n")

// ====== log2() ======
int log2(int x)
{
	int i;
	for(i=0;i<30;i++)
		if((1<<i)>x)
			break;
	return(i);
}

#pragma argsused
int main(int argc, char* argv[])
{
	// header
	printf("BMP to VHDL converter by Stanislav Maslan, s.maslan@seznam.cz, V1.00, 2012.05.05\n");

	// check parameters
	if(argc!=2)
	{
		usage();
		printf("Error - missing parameter!\n\n");
		return(1);
	}

	// try to create bitmap
	Graphics::TBitmap *bmp;
	try{
		bmp=new Graphics::TBitmap;
	}catch(Exception &exc){
		// failed
		usage();
		printf("Error - creating TBitmap failed with exception:\n\"%s\"\n\n",exc.Message.c_str());
		return(1);
	}

	// try to open source bitmap
	try{
		bmp->LoadFromFile(AnsiString(argv[1]));
	}catch(Exception &exc){
		// failed
		usage();
		printf("Error - opening image failed with exception:\n\"%s\"\n\n",exc.Message.c_str());
		return(1);
	}

	// check its format
	if(bmp->PixelFormat!=pf24bit)
	{
		// invalid format
		usage();
		printf("Error - invalid format, image must be 24-bit raster!\n\n");
		delete(bmp);
		return(1);
	}

	// get bitmap resolution
	int x=bmp->Width;
	int y=bmp->Height;

	// loose path extentsion
	char path[260];
	strcpy(path,argv[1]);
	char *pstr=strrchr(path,'.');
	if(pstr)
		*pstr='\0';

	// loose folders if some
	char c='\0';
	while(pstr>path && (c=*--pstr)!='\\' && c!='/')
		;
	if(c=='\\' || c=='/')
		pstr++;
	char fname[260];
	// store pure file name
	strcpy(fname,pstr);

	// generate destination path
	strcat(path,".vhd");

	// try to create destination file
	FILE *fw=fopen(path,"wt");
	if(!fw)
	{
		usage();
		printf("Error - creating destination file failed!\n\n");
		delete(bmp);
		return(1);
	}

	// print some file headers
	fprintf(fw,"-- BITMAP GENERATOR --\n");
	fprintf(fw,"-- Source bitmap: %s\n\n\n",argv[1]);

	fprintf(fw,"library IEEE;\n");
	fprintf(fw,"use IEEE.STD_LOGIC_1164.ALL;\n");
	fprintf(fw,"use IEEE.STD_LOGIC_ARITH.ALL;\n");
	fprintf(fw,"use IEEE.STD_LOGIC_UNSIGNED.ALL;\n\n");

	fprintf(fw,"package %s is\n",fname);
	fprintf(fw,"\t--- bitmap pixels ---\n");
	fprintf(fw,"\tfunction pixels (col: std_logic_vector; row: std_logic_vector) return std_logic;\n\n");
	fprintf(fw,"\t--- bitmap size ---\n");
	fprintf(fw,"\tconstant bmpx: integer := %d;\n",x);
	fprintf(fw,"\tconstant bmpy: integer := %d;\n",y);
	fprintf(fw,"\tconstant bitx: integer := %d;\n",log2(x));
	fprintf(fw,"\tconstant bity: integer := %d;\n",log2(y));
	fprintf(fw,"end %s;\n\n",fname);

	fprintf(fw,"package body %s is\n",fname);
	fprintf(fw,"\t--- bitmap pixels ---\n");
	fprintf(fw,"\tfunction pixels (col: std_logic_vector; row: std_logic_vector) return std_logic is\n");
	fprintf(fw,"\t\tvariable pix: std_logic;\n");
	fprintf(fw,"\tbegin\n");

	// convert image
	int i,j;
	for(j=0;j<y;j++)
	{
		uchar *pix=(uchar*)(bmp->ScanLine[j]);
		for(i=0;i<x;i++)
		{
			fprintf(fw,"\t\t%s (col=%-3d and row=%-3d) then pix := '%d';\n",(i!=0 || j!=0)?"elsif":"   if",i,j,*pix?0:1);
			pix+=3;
		}
	}

	// generate rest of VHDL trash
	fprintf(fw,"\t\telse pix := '0'; end if;\n\n");
	fprintf(fw,"\t\treturn pix;\n");
	fprintf(fw,"\tend pixels;\n");
	fprintf(fw,"end %s;\n",fname);

	// close destination file
	fclose(fw);

	// loose bitmap
	delete(bmp);

	// done
	printf("\ndone ...\n");

	return(0);
}
//---------------------------------------------------------------------------
